% spectrum analyzer
% arguments: 
% * signal:     input signal
% * rate.       sampling rate of input signal, Hz
% * windowBW:   spectrum smoothing done using brick wall filter with windowBW bandwidth
% * RBW:        resolution bandwidth RBW
%
% returned values:
% * fr:         spectrum
% * fb:         frequency base
% * use plot(fb, fr)
function [fr, fb]=mn_spectrum_local(signal, rate, windowBW, RBW)
  
% reverse spectrum
%signal=conj(signal);

% fr: "f"requency "r"esponse (plot y data)
% fb: "f"requency "b"ase (plot x data)
  l=length(signal);
  fb=mn_fftFreqbase(l, rate);
  fr=fft(signal);
  
  % Normalize amplitude to the number of samples, that contribute
  % to the spectrum
  fr=fr/l;
  
  fr=fr .* conj(fr);
  
  simLen=length(signal)/rate;
  fft_RBW=1/simLen;
    
  % Cyclic convolution with window:
  % Build window
  win=ones(1, l);
  offset=0;
  %  offset=1/simLen/2; % if even-length window is required,
  %  althoug asymmetrical
  win(find(fb < -windowBW/2+offset))=0;
  win(find(fb > windowBW/2+offset))=0;

  % Normalize total window to unity
  count=sum(win);
  if count < 1
    win(find(fb==0))=1;
    count=1;
  end
  win=win/count;

  % convolution in time domain => multiplication in frequency domain
  % Convert window to FD. Use fftshift to center the window around
  % FFT origin, so that the spectrum does not get shifted
  win=fft(fftshift(win));

  % fr to FD
  fr=fft(fr);
  % multiplication...
  fr=fr .* win;
  % back to TD
  fr=ifft(fr);
  % rotate 0 to center according to freqbase
  fr=fftshift(fr);

  % to dB, add small artificial noise floor
  fr=max(fr, 1e-17);
  fr=(10*log10(fr));

  % convert RBW
  fr=fr+10*log10(RBW/(fft_RBW));

  % ************************************
  % decimate data to reduce amount of plot points
  % The smallest visible detail is about windowBW wide,
  % due to filtering. A couple of samples in that width
  % are sufficient, more samples provide no advantage.
  % ************************************
 
  %incr=floor(l/4000);
  dataResolution_Hz=rate/l;
  incr=floor(windowBW/dataResolution_Hz/3);

  incr=1;
  if incr > 1 
    fr=fr(1:incr:l);
    fb=fb(1:incr:l);
  end  
  
  fb=fb/1e6; % use MHz
  